@@ -1,8 +1,12 @@
 #include "config.h"
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <errno.h>
-#include "pen.h"
+#include "conn.h"
+#include "diag.h"
+#include "event.h"
+#include "memory.h"
 #ifdef HAVE_KQUEUE
 #include <sys/types.h>
 #include <sys/event.h>
@@ -14,14 +18,6 @@ static int nfds, maxevents;
 static int count;
 static int pindex;
 
-#if 0
-static void kqueue_event_reset(void)
-{
-	DEBUG(2, "kqueue_event_reset()");
-        nfds = 0;
-}
-#endif
-
 static void kqueue_event_add(int fd, int events)
 {
 	DEBUG(2, "kqueue_event_add(fd=%d, events=%d)", fd, events);
@@ -74,8 +70,7 @@ static void kqueue_event_wait(void)
         count = kevent(kq, kev, nfds, kev_out, maxevents, &tv);
 	DEBUG(2, "kevent returns %d", count);
         if (count < 0 && errno != EINTR) {
-                perror("kevent");
-                error("Error on kevent");
+                error("Error on kevent: %s", strerror(errno));
         }
 	pindex = -1;
 	nfds = 0;
@@ -98,8 +93,7 @@ void kqueue_init(void)
 {
 	kq = kqueue();
 	if (kq == -1) {
-		perror("kqueue");
-		error("Error creating kernel queue");
+		error("Error creating kernel queue: %s", strerror(errno));
 	}
 	maxevents = connections_max*2+2;
 	kev = pen_malloc(maxevents*sizeof *kev);
