@@ -105,6 +105,7 @@ static char *webfile = NULL;
 static char listenport[1000];
 static int port;
 
+static int control_acl;
 static char *ctrlport = NULL;
 int listenfd;
 static int ctrlfd = -1;
@@ -834,7 +835,7 @@ static void usage(void)
 {
 	printf("usage:\n"
 	       "  pen [-C addr:port] [-X] [-b sec] [-c N] [-e host[:port]] \\\n"
-	       "	  [-t sec] [-x N] [-w dir] [-HPWadfhrs] \\\n"
+	       "	  [-t sec] [-x N] [-w dir] [-UHPWadfhrs] \\\n"
 	       "          [-o option] \\\n"
 #ifdef HAVE_LIBSSL
 	       "	  [-E certfile] [-K keyfile] \\\n"
@@ -848,6 +849,7 @@ static void usage(void)
 	       "  -T sec    tracking time in seconds (0 = forever) [%d]\n"
 	       "  -H	add X-Forwarded-For header in http requests\n"
 	       "  -U	use udp protocol support\n"
+	       "  -N	use hash for initial server selection without save server\n"
 	       "  -O option	use option in penctl format\n"
 	       "  -P	use poll() rather than select()\n"
 	       "  -Q    use kqueue to manage events (BSD)\n"
@@ -937,6 +939,7 @@ static void init(int argc, char **argv)
 	server = 0;
 
 	for (i = 1; i < argc; i++) {
+		DEBUG(2, "server[%d] = %s", server, argv[i]);
 		expand_servertable(server+1);
 		servers[server].status = 0;
 		servers[server].c = 0;	/* connections... */
@@ -947,6 +950,7 @@ static void init(int argc, char **argv)
 	}
 
 	if (e_server) {
+		DEBUG(2, "Emergency server = %s", e_server);
 		expand_servertable(EMERGENCY_SERVER+1);
 		emerg_server = EMERGENCY_SERVER;
 		servers[EMERGENCY_SERVER].status = 0;
@@ -958,6 +962,7 @@ static void init(int argc, char **argv)
 	}
 
 	if (a_server) {
+		DEBUG(2, "Abuse server = %s", a_server);
 		expand_servertable(ABUSE_SERVER+1);
 		abuse_server = ABUSE_SERVER;
 		servers[ABUSE_SERVER].status = 0;
@@ -1588,6 +1593,12 @@ static void do_cmd(char *b, void (*output)(void *, char *, ...), void *op)
 		} else {
 			tcp_fastclose = 0;
 		}
+	} else if (!strcmp(p, "tarpit_acl")) {
+		p = strtok(NULL, " ");
+		if (p) tarpit_acl = atoi(p);
+		if (tarpit_acl < -1 || tarpit_acl >= ACLS_MAX)
+			tarpit_acl = 0;
+		output(op, "tarpit_acl = %d", tarpit_acl);
 	} else if (!strcmp(p, "tcp_nodelay")) {
 		tcp_nodelay = 1;
 	} else if (!strcmp(p, "timeout")) {
@@ -1743,7 +1754,7 @@ static void add_client(int downfd, struct sockaddr_storage *cli_addr)
 		return;
 	}
 
-	conns[conn].initial = initial_server(conn);
+	conns[conn].initial = conns[conn].server = initial_server(conn);
 	if (conns[conn].initial == -1) {
 		DEBUG(1, "No initial server found, giving up");
 		close_conn(conn);
@@ -2239,9 +2250,9 @@ static int options(int argc, char **argv)
 	char b[1024];
 
 #ifdef HAVE_LIBSSL
-	char *opt = "B:C:F:O:S:T:b:c:e:i:j:l:m:o:p:q:t:u:w:x:DHPQWXUadfhnrsE:K:G:A:ZRL:";
+	char *opt = "B:C:F:O:S:T:b:c:e:i:j:l:m:o:p:q:t:u:w:x:DHNPQWXUadfhnrsE:K:G:A:ZRL:";
 #else
-	char *opt = "B:C:F:O:S:T:b:c:e:i:j:l:m:o:p:q:t:u:w:x:DHPQWXUadfhnrs";
+	char *opt = "B:C:F:O:S:T:b:c:e:i:j:l:m:o:p:q:t:u:w:x:DHNPQWXUadfhnrs";
 #endif
 
 	while ((c = getopt(argc, argv, opt)) != -1) {
@@ -2294,11 +2305,7 @@ static int options(int argc, char **argv)
 			blacklist_time = atoi(optarg);
 			break;
 		case 'c':
-#if 0
-			clients_max = atoi(optarg);
-#else
 			expand_clienttable(atoi(optarg));
-#endif
 			break;
 		case 'd':
 			debuglevel++;
@@ -2371,11 +2378,7 @@ static int options(int argc, char **argv)
 #endif
 			break;
 		case 'x':
-#if 0
-			connections_max = atoi(optarg);
-#else
 			expand_conntable(atoi(optarg));
-#endif
 			break;
 		case 'w':
 			webfile = pen_strdup(optarg);
